# Copyright (C) 2018 Adan Salazar <asalazargaribay@gmail.com>
#
#
# This file is part of GeoSentinel
#
#
# GeoSentinel is free software  you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# GeoSentinel is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with StereoVision.  If not, see <http://www.gnu.org/licenses/>.

"""
Utilities for user interaction with the "stereo" package.

Variables:

    * "SENTINEL_ARGUMENTS" - "argparse.ArgumentParser" for working with Sentinel products
    
Functions:

    * "find_files" - Discover Sentinel files in directory
    * "download_sentinel_mages" - Calibrate chessboard images discovered in a folder
    * "check_existing_directory" - Check out if the folder exist. Create folder if it does not exist
    * "select_products" -  # Select products that their intersection area is less than a certain percentage threshold

"""

from argparse import ArgumentParser

import os
import zipfile

from collections import OrderedDict

from osgeo import ogr

from sentinelsat.sentinel import SentinelAPI #read_geojson, geojson_to_wkt

from geosentinel.products import SentinelData


#from Sentinel.exceptions import ImageNotFoundError

#: Command line arguments for collecting information about Sentinel data and products
SENTINEL_ARGUMENTS = ArgumentParser(add_help=False)

SENTINEL_ARGUMENTS.add_argument("init_date", help="initial date for dowloading period.")

SENTINEL_ARGUMENTS.add_argument("end_date", help="final date for dowloading period.")

SENTINEL_ARGUMENTS.add_argument("product_type", help="sentinel product type {SLC, GRD, IW }")

SENTINEL_ARGUMENTS.add_argument("plattform", help="sentinel-1 or sentinel-2")

SENTINEL_ARGUMENTS.add_argument("overlap", type=int, help="Percentage of overlapping between the polygon " 
                        "provided by the user and the polygon of the image")  


SENTINEL_ARGUMENTS.add_argument("output_folder", help="Folder to download Sentinel files ")
    

SENTINEL_FIND_ARGUMENTS = ArgumentParser(add_help=False)

SENTINEL_FIND_ARGUMENTS.add_argument("folder", help="Folder to find Sentinel files ")

SENTINEL_FIND_ARGUMENTS.add_argument("plattform", help="Sentinel-1 or Sentinel-2")


SENTINEL_UNZIP_ARGUMENTS = ArgumentParser(add_help=False)

SENTINEL_UNZIP_ARGUMENTS.add_argument("folder", help="Folder to find Sentinel files")

SENTINEL_UNZIP_ARGUMENTS.add_argument("plattform", help="Sentinel-1 or Sentinel-2")

SENTINEL_UNZIP_ARGUMENTS.add_argument("product_type", help="sentinel product type {SM_SLC, SM_GRDF, SM_GRDH, SM_GRDM, IW_SLC, IW_GRDH, IW_GDRM, EW_SLC, EW_GRDH, EW_GRDM, WV_SLC, WV_GRDM, MSIL1C, OPER_PRD }")


def find_files(args):
    folder = args.folder
    satelite = args.plattform
    files = os.listdir(folder)
    if(satelite=="Sentinel-1"):
        namfil1 = 'S1A_'+ args.product_type
        namfil2 = 'S1B_'+ args.product_type
    if(satelite=="Sentinel-2"):
        namfil1 = 'S2A_'+ args.product_type
        namfil2 = 'S2B_'+ args.product_type
    files = [f for f in files if f.startswith(namfil1) or f.startswith(namfil2)]
    if(len(files)==0):
        print"Files not found..."
    return files

def unzip(list_files,args):
    for image in list_files:
        print"Unzziped... ",image
        File = args.folder + "\\" +image
        zip_ref = zipfile.ZipFile(File) # create zipfile object
        zip_ref.extractall(args.folder) # extract file to dir
        zip_ref.close() # close file

def check_existing_directory(folder) : 
    
    if not os.path.exists(folder):
        os.makedirs(folder)
        print folder + " was created"
    else: 
        print folder + " exists "


def select_products( products_list , products_df, user_footprint, threshold ) :
    
   
    products_down = OrderedDict()
    products_down = products_list.copy()
         
    polyfootprint = ogr.CreateGeometryFromWkt(user_footprint)
    areafootprint = polyfootprint.GetArea()
     
    for i in range(len(products_df)):
        
       inputpoly = ogr.CreateGeometryFromWkt(products_df.footprint[i])
        
       intersectionpoly = polyfootprint.Intersection(inputpoly)
        
       intersectionpoly_area = intersectionpoly.GetArea()
    
       por_intersection  = (intersectionpoly_area*100)/areafootprint
                
       if(por_intersection <=  threshold):
           # Deleting element
           del products_down[products_df.uuid[i]]

    return products_down


def download_sentinel_mages(args):
    
            
    check_existing_directory(args.output_folder)
    
    products = SentinelData()
    
    api = SentinelAPI(products.username, products.password, products.URL_Sentinel)
    
    products.interest_footprint =  args.footprint
    
    products.init_date = args.init_date
    
    products.end_date = args.end_date
    
    products.product_type = args.product_type
    
    products.platform_name = args.plattform
    
    products.intersection_th = args.overlap
    
        
    products_list = api.query(products.interest_footprint,
                     date = (products.init_date, products.end_date),
                     producttype = products.product_type,
                     platformname = products.platform_name)
    
    products_df = api.to_dataframe(products_list)
        
    products_to_downloading = select_products(products_list, products_df, products.interest_footprint, products.intersection_th ) 
        
   
    os.chdir(args.output_folder)
    api.download_all(products_to_downloading)


    

    
  

